from collections import OrderedDict

def read_labels(label_file, range_min, range_max):
    labs = {}
    labels = []
    try:
        of = open(label_file, "rt")
        labels = of.readlines()
    except IOError:
        print("skipped: "+label_file)
        return labs
    for line in labels:
        words = line.split()
        if (words[0] == "al"):
            adr = int(words[1], 16)
            sym = words[2]
            sym = sym.lstrip('.')
            if (sym[0] == '_' and sym[1] == '_'):
                continue # skip compiler internals
            if (adr >= range_min and adr <= range_max):
                if (adr in labs):
                    # multiple symbol
                    text = labs[adr]
                    textsplit = text.split()
                    if (sym not in textsplit):
                        text = text + " " + sym
                        labs[adr] = text
                else:
                    labs[adr] = sym
    return labs    

def label_to_nl(label_file, nl_file, range_min, range_max):
    labs = read_labels(label_file, range_min, range_max)
    sout = ""
    for (adr, sym) in labs.items():
        sout += ("$%04X#%s#\n" % (adr, sym))
    open(nl_file, "wt").write(sout)
    print("debug symbols: " + nl_file)

def label_to_wch(label_file, wch_file, range_min, range_max):
    labs = read_labels(label_file, range_min, range_max)
    labels = []
    sout = ""
    labs[range_max+1] = "END"
    sout += "\n"
    sout += "%d\n" % (1+range_max-range_min)
    last_adr = 0x0000
    last_sym = "NONE"
    count = 0
    labs = OrderedDict(sorted(labs.items(), key=lambda t: t[0]))
    for (adr, sym) in labs.items():
        # print ("> %04X = %s" % (adr,sym))
        l = adr-last_adr
        if l == 1:
            sout += "%05X\t%04X\tb\th\t0\t%s\n" % (count, last_adr, last_sym)
            count += 1
        else:
            subcount = 0
            for a in range(last_adr,adr):
                sout += "%05X\t%04X\tb\th\t0\t%s+%X\n" % (count, a, last_sym, subcount)
                subcount += 1
                count += 1
        last_sym = sym
        last_adr = adr
    open(wch_file, "wt").write(sout)
    print("watch: " + wch_file)

def stat_label(l):
    if (l.find("stat") != -1):
        return True
    return False

def label_to_stats(label_file, stat_file, range_min, range_max):
    labs = read_labels(label_file, range_min, range_max)
    labs[range_max+1] = "END"
    labs = OrderedDict(sorted(labs.items(), key=lambda t: t[0]))
    sout = ""
    last_addr = range_min
    last_addr2 = range_min
    for (adr, sym) in labs.items():
        diff = adr - last_addr
        diff2 = adr - last_addr2
        stat = stat_label(sym)
        sv = 0
        if stat:
            words = sym.split()
            stat_sym = ""
            for w in words:
                if stat_label(w):
                    stat_sym = stat_sym + " " + w
            if (len(stat_sym) > 0):
                stat_sym = stat_sym[1:]
            sv = 1
            sout += ("%8d, %8d, %d, $%04X, %s\n" % (diff,diff2,sv,adr,stat_sym))
        last_addr = adr
        if stat:
            last_addr2 = adr
    open(stat_file, "wt").write(sout)
    print("code stats: " + stat_file)


    
if __name__ == "__main__":
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.ram.nl", 0x0000, 0x7FFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.0.nl", 0x8000, 0xFFFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.1.nl", 0x8000, 0xFFFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.2.nl", 0x8000, 0xFFFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.3.nl", 0x8000, 0xFFFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.4.nl", 0x8000, 0xFFFF)
    label_to_nl("temp\\st_nsf.labels.txt", "temp\\st_nsf.nsf.5.nl", 0x8000, 0xFFFF)    
